//======================================================================
//	File: KPUApp.cc
//----------------------------------------------------------------------
//	Copyright © 1997-1998 KISAKA Toshihiro.
//		kisaka@pochi.iijnet.or.jp
//======================================================================

#include "KPUApp.h"

//	System Headers
#include <Debug.h>
#include <Screen.h>

//	Project Headers
#include "PullUp.h"
#include "KPUWindow.h"
#include "KPUView.h"


//======================================================================
//	Class: KPUApp
//======================================================================

KPUApp* my_app = NULL;

//----------------------------------------------------------------------
//	KPUApp()
//----------------------------------------------------------------------

KPUApp::KPUApp()
	: BApplication(kSignature),
	  mHotRect(0.0, 0.0, -1.0, -1.0)
{
	my_app = this;
}


//----------------------------------------------------------------------
//	~KPUApp()
//----------------------------------------------------------------------

KPUApp::~KPUApp()
{
}


//----------------------------------------------------------------------
//	Pulse()
//----------------------------------------------------------------------

void
KPUApp::MessageReceived(BMessage* message)
{
	switch (message->what)
	{
	  case K_UPDATE_HOTRECT:
		this->UpdateHotRect();
		break;
	  default:
		BApplication::MessageReceived(message);
		break;
	}
}


//----------------------------------------------------------------------
//	Pulse()
//----------------------------------------------------------------------

void
KPUApp::Pulse()
{
	this->UpdateHotRect();
}


//----------------------------------------------------------------------
//	ReadyToRun()
//----------------------------------------------------------------------

void
KPUApp::ReadyToRun()
{
	this->SetPulseRate(kCheckRateDeskbar);
	this->UpdateHotRect();

	KPUWindow* aWindow = NULL;
	KPUView* aView = NULL;

	aWindow = new KPUWindow();
	aWindow->SetPulseRate(kCheckRateMouse);

	aView = new KPUView();
	aWindow->AddChild(aView);

	aWindow->Show();
	aWindow->Hide();
}


//----------------------------------------------------------------------
//	BRect DeskbarFrame() const
//----------------------------------------------------------------------

BRect
KPUApp::DeskbarFrame() const
{
	const bigtime_t kTimeout = 1000000;	// 1 sec.
	const BRect kInvalidRect(0, 0, -1, -1);

	status_t aErr = B_OK;
	BRect aResultRect;

	BMessenger aMessenger(kSigDeskbar);
	if (!aMessenger.IsValid())
		return kInvalidRect;

	BMessage aMessage(B_GET_PROPERTY);
	aErr = aMessage.AddSpecifier("Frame");
	aErr = aMessage.AddSpecifier("Window", "Deskbar");

	BMessage aReply;
	aErr = aMessenger.SendMessage(&aMessage, &aReply, kTimeout, kTimeout);
	if (aErr < B_OK)
		return kInvalidRect;

	switch (aReply.what)
	{
	  case B_REPLY:
		aErr = aReply.FindRect("result", &aResultRect);
		if (aErr < B_OK)
			aResultRect = kInvalidRect;
		break;
	  default:
		aResultRect = kInvalidRect;
		break;
	}

	PRINT_OBJECT(aResultRect);

	return aResultRect;
}


//----------------------------------------------------------------------
//	UpdateHotRect()
//----------------------------------------------------------------------

void
KPUApp::UpdateHotRect()
{
	BRect aHotRect = this->DeskbarFrame();
	if (!aHotRect.IsValid())
		return;

	BRect aScreenFrame = BScreen().Frame();

	PRINT_OBJECT(aScreenFrame);

	if (aHotRect.left != aScreenFrame.left)
		aHotRect.left = aHotRect.right;

	if (aHotRect.top != aScreenFrame.top)
		aHotRect.top = aHotRect.bottom;

	if (aHotRect.right != aScreenFrame.right)
		aHotRect.right = aHotRect.left;

	if (aHotRect.bottom != aScreenFrame.bottom)
		aHotRect.bottom = aHotRect.top;

	PRINT_OBJECT(aHotRect);

	this->mHotRect = aHotRect;
}
